<?php

namespace Drupal\acquia_connector\Client;

use Acquia\ContentHubClient\CDF\CDFObject;
use Acquia\ContentHubClient\Event\GetCDFTypeEvent;
use Acquia\Hmac\Guzzle\HmacAuthMiddleware;
use Acquia\Hmac\Key;
use Drupal\acquia_connector\Settings;
use GuzzleHttp\Client;
use GuzzleHttp\HandlerStack;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Class ObjectFactory.
 *
 * @package Acquia\ContentHubClient
 * @codeCoverageIgnore
 */
class ObjectFactory {

  /**
   * Creates Guzzle client.
   *
   * @param array $config
   *   Initial data.
   *
   * @return \GuzzleHttp\Client
   *   GuzzleClient instance.
   */
  public static function getGuzzleClient(array $config): Client {
    return new Client($config);
  }

  /**
   * Creates authentication key.
   *
   * @param string $api_key
   *   API key.
   * @param string $secret
   *   Secret.
   *
   * @return \Acquia\Hmac\Key
   *   Key instance.
   */
  public static function getAuthenticationKey($api_key, $secret): Key {
    return new Key($api_key, $secret);
  }

  /**
   * Creates HmacAuthMiddleware.
   *
   * @param \Acquia\Hmac\Key $key
   *   Key instance.
   *
   * @return \Acquia\Hmac\Guzzle\HmacAuthMiddleware
   *   HmacAuthMiddleware instance.
   */
  public static function getHmacAuthMiddleware(Key $key): HmacAuthMiddleware {
    return new HmacAuthMiddleware($key);
  }

  /**
   * Creates Settings object instance.
   *
   * @param string $name
   *   Name of settings group.
   * @param string $uuid
   *   The assigned UUID.
   * @param string $api_key
   *   API key.
   * @param string $secret
   *   Secret key.
   * @param string $url
   *   The URL of the end point to consult.
   * @param string|null $shared_secret
   *   Shared secret.
   * @param array $webhook
   *   Webhook UUID.
   *
   * @return \Acquia\ContentHubClient\Settings
   *   Settings object instance.
   */
  public static function instantiateSettings(string $name, string $uuid, string $api_key, string $secret, string $url, ?string $shared_secret = NULL, array $webhook = []): Settings {
    return new Settings($name, $uuid, $api_key, $secret, $url, $shared_secret);
  }

  /**
   * Creates a default handler stack that can be used by clients.
   *
   * @return \GuzzleHttp\HandlerStack
   *   HandlerStack instance.
   */
  public static function getHandlerStack(): HandlerStack {
    return HandlerStack::create();
  }

  /**
   * Creates ContentHubClient instance.
   *
   * @param array $config
   *   Config.
   * @param \Psr\Log\LoggerInterface $logger
   *   Logger instance.
   * @param \Acquia\ContentHubClient\Settings $settings
   *   Settings instance.
   * @param \Acquia\Hmac\Guzzle\HmacAuthMiddleware $middleware
   *   HmacAuthMiddleware instance.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
   *   Event dispatcher instance.
   * @param string $api_version
   *   API version.
   *
   * @return \Acquia\ContentHubClient\ContentHubClient
   *   ContentHubClient instance.
   */
  public static function getAcquiaConnectorClient( // phpcs:ignore
    array $config,
    LoggerInterface $logger,
    Settings $settings,
    HmacAuthMiddleware $middleware,
    EventDispatcherInterface $dispatcher,
  ): AcquiaConnectorClient {
    return new AcquiaConnectorClient($config, $logger, $settings, $middleware,
      $dispatcher);
  }

}
