<?php

namespace Drupal\acquia_connector\EventSubscriber\GetSettings;

use Drupal\acquia_connector\AcquiaConnectorEvents;
use Drupal\acquia_connector\Event\AcquiaSubscriptionSettingsEvent;
use Drupal\acquia_connector\Settings;
use Drupal\acquia_connector\SiteProfile;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Gets the ContentHub Server settings from environment variable.
 */
class FromAcquiaCloud implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * Array containing the necessary environment variable keys.
   */
  const ENVIRONMENT_VARIABLES = [
    'AH_SITE_ENVIRONMENT',
    'AH_SITE_NAME',
    'AH_SITE_GROUP',
    'AH_APPLICATION_UUID'
  ];

  /**
   * Acquia Connector logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Drupal messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Site Profile Service.
   *
   * @var \Drupal\acquia_connector\SiteProfile
   */
  protected $siteProfile;

  /**
   * GetSettingsFromEnvVarTest constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   Acquia Connector logger channel.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Drupal messenger interface.
   */
  public function __construct(LoggerChannelInterface $logger, MessengerInterface $messenger, SiteProfile $site_profile) {
    $this->logger = $logger;
    $this->messenger = $messenger;
    $this->siteProfile = $site_profile;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[AcquiaConnectorEvents::GET_SETTINGS][] = ['onGetSettings', 100];
    return $events;
  }

  /**
   * Extract settings from environment and create a Settings object.
   *
   * @param \Drupal\acquia_connector\Event\AcquiaSubscriptionSettingsEvent $event
   *   The dispatched event.
   *
   * @see \Acquia\ContentHubClient\Settings
   */
  public function onGetSettings(AcquiaSubscriptionSettingsEvent $event) {
    $metadata = [];
    // Replace with getenv() once platform is updated in DIT-185.
    foreach (self::ENVIRONMENT_VARIABLES as $var) {
      if (!empty($_ENV[$var])) {
        $metadata[$var] = $_ENV[$var];
      }
    }
    // If the expected Acquia cloud environment variables are missing, return.
    if (count($metadata) !== count(self::ENVIRONMENT_VARIABLES)) {
      return;
    }

    global $config;
    $site_name = $this->siteProfile->checkAcquiaHosted() ? $metadata['AH_SITE_ENVIRONMENT'] . '_' . $metadata['AH_SITE_NAME'] : '';
    // Mock up a subscription array to pass in the UUID.
    $sub['uuid'] = $metadata['AH_APPLICATION_UUID'];
    $site_machine_name = $this->siteProfile->getMachineName($sub);
    $key = $config['ah_network_key'];
    $identifier = $config['ah_network_identifier'];

    $settings = new Settings(
      $event->getConfig(),
      $identifier,
      $key,
      $site_name,
      $site_machine_name,
      $metadata
    );

    $event->setProvider('acquia_cloud');
    $event->setSettings($settings);
    $event->stopPropagation();
  }

}
