<?php

namespace Drupal\acquia_connector;

use Acquia\Hmac\Guzzle\HmacAuthMiddleware;
use Acquia\Hmac\Key;
use Drupal\Core\Config\Config;

/**
 * Acquia Subscription Settings.
 *
 * Single centralized place for accessing and updating Acquia Connector
 * settings. All currently existing configs should be moved here and use Drupal
 * State API instead of Drupal Config. For more info visit
 * https://www.drupal.org/node/2635138.
 */
class Settings {

  /**
   * The user friendly site name.
   *
   * @var string
   */
  protected $siteName;

  /**
   * Machine name for site name.
   *
   * @var string
   */
  protected $machineName;

  /**
   * Acquia Network ID
   *
   * eg: ABCD-12345
   *
   * @var string
   */
  protected $identifier;

  /**
   * The shared secret key.
   *
   * @var string
   */
  protected $secretKey;

  /**
   * The endpoint to access subscription data.
   *
   * @var string
   */
  protected $url;

  /**
   * The shared secret.
   *
   * @var string|null
   */
  protected $sharedSecret;

  /**
   * Subscription Data.
   *
   * @var array
   */
  protected $subscriptionData;

  /**
   * Config object from `acquia_connector.settings`
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * Readonly status of the Settings object.
   *
   * @var bool
   */
  protected $readonly = TRUE;

  /**
   * Additional Metadata provided by some Settings providers.
   *
   * @var array|mixed
   */
  protected $metadata;

  /**
   * Constructs a Settings object.
   *
   * These settings have a null option to handle initial setup through the
   * ClientFactory. At that point, only config is required.
   *
   * @param \Drupal\Core\Config\Config $config
   *   Static config for Acquia Connector
   * @param string $network_id
   *   Site Machine Name
   * @param string $secret_key
   *   Secret key.
   * @param string $name
   *   Site Name
   * @param string $machine_name
   *   Site Machine Name
   */
  public function __construct(Config $config, string $network_id = NULL, string $secret_key = NULL,  string $name = NULL, string $machine_name = NULL, $metadata = NULL) {
    $this->config = $config;
    $this->siteName = $name ?? '';
    $this->machineName = $machine_name ?? '';
    $this->identifier = $network_id ?? ''; // eg: ABCD-2345032
    $this->secretKey = $secret_key ?? '';
    $this->metadata = $metadata ?? [];
  }

  /**
   * Returns Acquia Subscription identifier.
   *
   * @return mixed
   *   Acquia Subscription identifier.
   */
  public function getIdentifier() {
    return $this->identifier ?? NULL;
    //  return \Drupal::state()->get('acquia_connector.identifier');
  }

  /**
   * Returns Acquia Subscription key.
   *
   * @return mixed
   *   Acquia Subscription key.
   */
  public function getSecretKey() {
    return $this->secretKey ?? NULL;
    //return \Drupal::state()->get('acquia_connector.key');
  }

  /**
   * Returns site name
   *
   * @return mixed
   *   Acquia Site Name.
   */
  public function getSiteName() {
    return $this->siteName ?? NULL;
    //return \Drupal::state()->get('acquia_connector.key');
  }

  /**
   * Returns machine name of the site.
   *
   * @return mixed
   *   Acquia Site Name.
   */
  public function getMachineName() {
    return $this->machineName ?? NULL;
    //return \Drupal::state()->get('acquia_connector.key');
  }

  /**
   * Retrieves Subscription data from site's State.
   *
   * @return array
   */
  public function getSubscriptionData() {
    return \Drupal::state()->get('acquia_subscription_data') ?? [];
  }

  /**
   * Returns URL of the Acquia Subscription API.
   *
   * @return string
   *   URL of the endpoint.
   */
  public function getApiUrl() {
    return $this->config->get('spi.server') ?? 'https://nspi.acquia.com';
  }

  /**
   * Returns static connector config settings
   *
   * @return \Drupal\Core\Config\Config
   *   Acquia Connector Config Object.
   */
  public function getConfig() {
    return $this->config;
  }

  /**
   * Returns the metadata array, or a specific piece of metadata if it exists.
   *
   * @param $key
   *   Metadata key.
   *
   * @return mixed
   */
  public function getMetadata($key = NULL) {
    if (isset($key) && isset($this->metadata[$key])) {
      return $this->metadata[$key];
    }
    else if (isset($key)) {
      return [];
    }
    else {
      return $this->metadata;
    }
  }

  /**
   * Deletes all stored data.
   */
  public function deleteAllData() {
    \Drupal::state()->deleteMultiple([
      'acquia_connector.key',
      'acquia_connector.identifier',
      'spi.site_name',
      'spi.site_machine_name',
      'acquia_subscription_data',
    ]);
  }

  /**
   * Gets readonly status for the settings object.
   *
   * @return bool
   *   Readonly Status.
   */
  public function getReadonly() {
    return $this->readonly;
  }

  /**
   * Sets readonly status for the settings object.
   */
  public function setReadOnly($readonly) {
    $this->readonly = $readonly;
  }
}
