<?php

namespace Drupal\acquia_connector;

use Drupal\acquia_connector\Client\ClientFactory;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Database\Database;
use Drupal\Core\Http\RequestStack;
use Drupal\Core\State\State;

/**
 * Site Profile methods
 *
 * @package Drupal\acquia_contenthub
 */
class SiteProfile {

  /**
   * The Drupal Request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * Drupal State Service.
   *
   * @var \Drupal\Core\State\State
   */
  protected $state;

  /**
   * Acquia Connector Site Profile constructor.
   *
   * @param \Drupal\Core\Http\RequestStack $request_stack
   *   The Request Stack.
   * @param \Drupal\Core\State\State $state
   *   The state object
   */
  public function __construct(RequestStack $request_stack, State $state) {
    $this->request = $request_stack->getCurrentRequest();
    $this->state = $state->get('acquia_connector.settings');
  }

  /**
   * Attempt to determine if this site is hosted with Acquia.
   *
   * @return bool
   *   TRUE if site is hosted with Acquia, otherwise FALSE.
   */
  public function checkAcquiaHosted() {
    return isset($_SERVER['AH_SITE_ENVIRONMENT'], $_SERVER['AH_SITE_NAME']);
  }

  /**
   * Returns a unique string built on the current domain.
   *
   * @return string
   */
  public function getNonAcquiaSiteId() {
    $base_url = $this->request->getHost();
    return $base_url . '_' . substr(md5(uniqid(mt_rand(), true)), 0, 8);
  }

  /**
   * Generate the site name for connector.
   *
   * @return string
   *   The Acquia Hosted name.
   */
  public function getSiteName($subscription_name) {
    // Acquia Hosted
    if ($this->checkAcquiaHosted() && $subscription_name) {
      return $subscription_name . ': ' . $_SERVER['AH_SITE_ENVIRONMENT'];
    }
    // Locally Hosted
    return $subscription_name . ': ' . $this->getNonAcquiaSiteId();

  }

  /**
   * Generate the machine name for connector
   *
   * @return string
   *   The suggested Acquia Hosted machine name.
   */
  public function getMachineName($subscription_data): string {
    if (is_array($subscription_data)) {
      $sub_uuid = str_replace('-', '_', $this->getIdFromSub($subscription_data));

      if ($this->checkAcquiaHosted()) {
        return $sub_uuid . '__' . $_SERVER['AH_SITE_NAME'] . '__' . uniqid();
      }
      return $sub_uuid . '__' . str_replace('.', '_', $this->getNonAcquiaSiteId());
    }
    return '';
  }

  /**
   * Gets the subscription UUID from subscription data.
   *
   * @param array $sub_data
   *   An array of subscription data.
   *
   * @see acquia_agent_settings('acquia_subscription_data')
   *
   * @return string
   *   The UUID taken from the subscription data.
   */
  public function getIdFromSub(array $sub_data) {
    if (!empty($sub_data['uuid'])) {
      return $sub_data['uuid'];
    }

    // Otherwise, get this form the sub url.
    $url = UrlHelper::parse($sub_data['href']);
    $parts = explode('/', $url['path']);
    // Remove '/dashboard'.
    array_pop($parts);

    return end($parts);
  }

  /**
   * Check if a site environment change has been detected.
   *
   * @return bool
   *   TRUE if change detected that needs to be addressed, otherwise FALSE.
   */
  public function checkEnvironmentChange() {
    $changes = $this->state->get('spi.environment_changes');
    $change_action = $this->state->get('spi.environment_changed_action');

    return !empty($changes) && empty($change_action);
  }

  /**
   * Attempt to determine the version of Drupal being used.
   *
   * Note, there is better information on this in the common.inc file.
   *
   * @return array
   *   An array containing some detail about the version
   */
  private function getVersionInfo() {
    $server = $this->request->server->all();
    $ver = [];

    $ver['base_version'] = \Drupal::VERSION;
    $install_root = DRUPAL_ROOT;
    $ver['distribution'] = '';

    return $ver;
  }

  /**
   * Gather platform specific information.
   *
   * @return array
   *   An associative array keyed by a platform information type.
   */
  public function getPlatform() {
    $server = $this->request->server;
    // Database detection depends on the structure starting with the database.
    $db_class = '\Drupal\Core\Database\Driver\\' . Database::getConnection()->driver() . '\Install\Tasks';
    $db_tasks = new $db_class();
    // Webserver detection is based on name being before the slash, and
    // version being after the slash.
    preg_match('!^([^/]+)(/.+)?$!', $server->get('SERVER_SOFTWARE'), $webserver);

    if (isset($webserver[1]) && stristr($webserver[1], 'Apache') && function_exists('apache_get_version')) {
      $webserver[2] = apache_get_version();
    }

    // Get some basic PHP vars.
    $php_quantum = [
      'memory_limit' => ini_get('memory_limit'),
      'register_globals' => 'Off',
      'post_max_size' => ini_get('post_max_size'),
      'max_execution_time' => ini_get('max_execution_time'),
      'upload_max_filesize' => ini_get('upload_max_filesize'),
      'error_log' => ini_get('error_log'),
      'error_reporting' => ini_get('error_reporting'),
      'display_errors' => ini_get('display_errors'),
      'log_errors' => ini_get('log_errors'),
      'session.cookie_domain' => ini_get('session.cookie_domain'),
      'session.cookie_lifetime' => ini_get('session.cookie_lifetime'),
      'newrelic.appname' => ini_get('newrelic.appname'),
      'sapi' => php_sapi_name(),
    ];

    $platform = [
      'php'               => PHP_VERSION,
      'webserver_type'    => isset($webserver[1]) ? $webserver[1] : '',
      'webserver_version' => isset($webserver[2]) ? $webserver[2] : '',
      'php_extensions'    => get_loaded_extensions(),
      'php_quantum'       => $php_quantum,
      'database_type'     => (string) $db_tasks->name(),
      'database_version'  => Database::getConnection()->version(),
      'system_type'       => php_uname('s'),
      // php_uname() only accepts one character, so we need to concatenate
      // ourselves.
      'system_version'    => php_uname('r') . ' ' . php_uname('v') . ' ' . php_uname('m') . ' ' . php_uname('n'),
    ];

    return $platform;
  }

  /**
   * Gather information about modules on the site.
   *
   * @return array
   *   An associative array keyed by filename of associative arrays with
   *   information on the modules.
   */
  private function getModules() {
    $modules = \Drupal::service('extension.list.module')->reset()->getList();
    uasort($modules, 'system_sort_modules_by_info_name');

    $result = [];
    $keys_to_send = ['name', 'version', 'package', 'core', 'project'];
    foreach ($modules as $module) {
      $info = [];
      $info['status'] = $module->status;
      foreach ($keys_to_send as $key) {
        $info[$key] = isset($module->info[$key]) ? $module->info[$key] : '';
      }
      $info['filename'] = $module->getPathname();
      if (empty($info['project']) && $module->origin == 'core') {
        $info['project'] = 'drupal';
      }

      $result[] = $info;
    }
    return $result;
  }

}
